@@ -43,6 +43,7 @@
  
 #define COMMANDLINE_DEFAULT_ICON_SIZE 6
 
+static gboolean icons_initialized = FALSE;
 static GtkIconSize button_icon_size = 0;
 
 static const BonoboUIVerb mini_commander_menu_verbs [] = {
@@ -89,6 +90,9 @@ command_line_init_stock_icons ()
 
     GtkIconFactory *factory;
 
+    if (icons_initialized)
+	    return;
+
     factory = gtk_icon_factory_new ();
     gtk_icon_factory_add_default (factory);
 
@@ -97,6 +101,8 @@ command_line_init_stock_icons ()
     button_icon_size = gtk_icon_size_register ("mini-commander-icon",
                                                  COMMANDLINE_DEFAULT_ICON_SIZE,
                                                  COMMANDLINE_DEFAULT_ICON_SIZE);
+
+    icons_initialized = TRUE;
     g_object_unref (factory);
 
 }
@@ -341,6 +347,31 @@ static gboolean
 mini_commander_applet_fill (PanelApplet *applet)
 {
     MCData *mc;
+    GConfClient *client;
+
+    client = gconf_client_get_default ();
+    if (gconf_client_get_bool (client, "/desktop/gnome/lockdown/inhibit_command_line", NULL)) {
+	    GtkWidget *error_dialog;
+
+	    error_dialog = gtk_message_dialog_new (NULL,
+						   GTK_DIALOG_DESTROY_WITH_PARENT,
+						   GTK_MESSAGE_ERROR,
+						   GTK_BUTTONS_OK,
+						   _("Command line has been disabled by your system administrator"));
+
+	    gtk_window_set_resizable (GTK_WINDOW (error_dialog), FALSE);
+	    gtk_dialog_set_has_separator (GTK_DIALOG (error_dialog), FALSE);
+	    gtk_window_set_screen (GTK_WINDOW (error_dialog),
+				   gtk_widget_get_screen (GTK_WIDGET (applet)));
+	    gtk_dialog_run (GTK_DIALOG (error_dialog));
+	    gtk_widget_destroy (error_dialog);
+
+	    /* Note that this is only kosher if this is an out of process thing,
+	       which we really are.  We really don't need/want this applet when
+	       command line is disabled */
+	    exit (1);
+    }
+
     
     gnome_window_icon_set_default_from_file (GNOME_ICONDIR "/gnome-mini-commander.png");
     
@@ -375,6 +406,17 @@ mini_commander_applet_fill (PanelApplet *applet)
 				       mini_commander_menu_verbs,
 				       mc);
 
+    if (panel_applet_get_locked_down (mc->applet)) {
+	    BonoboUIComponent *popup_component;
+
+	    popup_component = panel_applet_get_popup_component (mc->applet);
+
+	    bonobo_ui_component_set_prop (popup_component,
+					  "/commands/Props",
+					  "hidden", "1",
+					  NULL);
+    }
+
     set_atk_name_description (GTK_WIDGET (applet),
 			      _("Mini-Commander applet"),
 			      _("This applet adds a command line to the panel"));
